local gen2d = require 'poisson'

local cmode="norm"

function love.load()
	stage = {w=love.graphics.getWidth(), h=love.graphics.getHeight()}
	spacing = 10

	local points=gen2d(stage.w-20,stage.h-20,10)
	
	vertices = {}
	for _,p in ipairs(points) do
		table.insert(vertices,{offset=0,angle=0,x=math.floor(p.x)+10,y=math.floor(p.y)+10})
	end

	cur_time = 0
end

function love.keypressed(key)
	if key == "tab" then
		if cmode=="norm" then
			cmode="diff"
		elseif cmode=="diff" then
			cmode="expr"
		else
			cmode="norm"
		end
	end
end

function love.update(dt)
	cur_time = cur_time + dt
	for _,vert in ipairs(vertices) do
			vert.offset = (love.math.noise(vert.x/200, vert.y/200, cur_time/2))*1
			vert.angle = (love.math.noise(vert.x/200, vert.y/200, cur_time/2-20))*2*math.pi
	end
end

-- Converts HSL to RGB. (input and output range: 0 - 255)
function HSL(h, s, l, a)
    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

function love.draw()
	love.graphics.setColor(0, 0, 0)
	love.graphics.rectangle("fill", 0, 0, stage.w, stage.h)
	love.graphics.setColor(255, 255, 255)
	local alias=255
	if cmode=="diff" then alias=128 end
	-- Color grid
	if cmode ~= "norm" then
		for _,vert in ipairs(vertices) do
			love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), alias))
			love.graphics.circle("fill", vert.x, vert.y, 10)
		end
	end
	for _,vert in ipairs(vertices) do
		love.graphics.setColor(HSL(vert.angle/(2*math.pi)*255, 255, 255*(vert.offset), alias))
		love.graphics.circle("fill", vert.x, vert.y, 5)
	end


end